package javax.swing;

import javax.swing.event.DocumentEvent;
import javax.swing.text.JTextComponent;
import java.awt.event.ActionEvent;
import javax.swing.text.AbstractDocument;
import javax.swing.text.NavigationFilter;
import javax.swing.text.DocumentFilter;
import java.io.Serializable;
import javax.swing.text.InternationalFormatter;
import javax.swing.text.DateFormatter;
import java.text.DecimalFormat;
import javax.swing.text.NumberFormatter;
import javax.swing.text.DefaultFormatter;
import java.util.Date;
import java.text.NumberFormat;
import java.text.DateFormat;
import javax.swing.plaf.UIResource;
import java.io.IOException;
import java.io.ObjectOutputStream;
import javax.swing.text.Document;
import javax.swing.text.TextAction;
import java.awt.im.InputContext;
import java.awt.EventQueue;
import java.awt.event.FocusEvent;
import java.text.AttributedCharacterIterator;
import java.awt.event.InputMethodEvent;
import java.text.ParseException;
import javax.swing.text.DefaultFormatterFactory;
import java.text.Format;
import javax.swing.event.DocumentListener;

public class JFormattedTextField extends JTextField
{
    private static final String uiClassID = "FormattedTextFieldUI";
    private static final Action[] defaultActions;
    public static final int COMMIT = 0;
    public static final int COMMIT_OR_REVERT = 1;
    public static final int REVERT = 2;
    public static final int PERSIST = 3;
    private JFormattedTextField.AbstractFormatterFactory factory;
    private JFormattedTextField.AbstractFormatter format;
    private Object value;
    private boolean editValid;
    private int focusLostBehavior;
    private boolean edited;
    private DocumentListener documentListener;
    private Object mask;
    private ActionMap textFormatterActionMap;
    private boolean composedTextExists;
    private JFormattedTextField.FocusLostHandler focusLostHandler;
    
    public JFormattedTextField() {
        this.composedTextExists = false;
        this.enableEvents(4L);
        this.setFocusLostBehavior(1);
    }
    
    public JFormattedTextField(Object value) {
        this();
        this.setValue(value);
    }
    
    public JFormattedTextField(Format format) {
        this();
        this.setFormatterFactory(this.getDefaultFormatterFactory(format));
    }
    
    public JFormattedTextField(JFormattedTextField.AbstractFormatter defaultFormat) {
        this(new DefaultFormatterFactory(defaultFormat));
    }
    
    public JFormattedTextField(JFormattedTextField.AbstractFormatterFactory formatterFactory) {
        this();
        this.setFormatterFactory(formatterFactory);
    }
    
    public JFormattedTextField(JFormattedTextField.AbstractFormatterFactory formatterFactory, Object o) {
        this(o);
        this.setFormatterFactory(formatterFactory);
    }
    
    public void setFocusLostBehavior(int focusLostBehavior) {
        if (focusLostBehavior != 0 && focusLostBehavior != 1 && focusLostBehavior != 3 && focusLostBehavior != 2) {
            throw new IllegalArgumentException("setFocusLostBehavior must be one of: JFormattedTextField.COMMIT, JFormattedTextField.COMMIT_OR_REVERT, JFormattedTextField.PERSIST or JFormattedTextField.REVERT");
        }
        this.focusLostBehavior = focusLostBehavior;
        return;
    }
    
    public int getFocusLostBehavior() {
        return this.focusLostBehavior;
    }
    
    public void setFormatterFactory(JFormattedTextField.AbstractFormatterFactory factory) {
        this.firePropertyChange("formatterFactory", this.factory, this.factory = factory);
        this.setValue(this.getValue(), true, false);
    }
    
    public JFormattedTextField.AbstractFormatterFactory getFormatterFactory() {
        return this.factory;
    }
    
    protected void setFormatter(JFormattedTextField.AbstractFormatter abstractFormatter) {
        JFormattedTextField.AbstractFormatter format = this.format;
        if (format != null) {
            format.uninstall();
        }
        this.setEditValid(true);
        if ((this.format = abstractFormatter) != null) {
            abstractFormatter.install(this);
        }
        this.setEdited(false);
        this.firePropertyChange("textFormatter", format, abstractFormatter);
    }
    
    public JFormattedTextField.AbstractFormatter getFormatter() {
        return this.format;
    }
    
    public void setValue(Object o) {
        if (o != null && this.getFormatterFactory() == null) {
            this.setFormatterFactory(this.getDefaultFormatterFactory(o));
        }
        this.setValue(o, true, true);
    }
    
    public Object getValue() {
        return this.value;
    }
    
    public void commitEdit() throws ParseException {
        JFormattedTextField.AbstractFormatter formatter = this.getFormatter();
        if (formatter != null) {
            this.setValue(formatter.stringToValue(this.getText()), false, true);
        }
    }
    
    private void setEditValid(boolean b) {
        if (b != this.editValid) {
            this.editValid = b;
            this.firePropertyChange("editValid", Boolean.valueOf(!b), Boolean.valueOf(b));
        }
    }
    
    public boolean isEditValid() {
        return this.editValid;
    }
    
    protected void invalidEdit() {
        UIManager.getLookAndFeel().provideErrorFeedback(this);
    }
    
    @Override
    protected void processInputMethodEvent(InputMethodEvent e) {
        AttributedCharacterIterator text = e.getText();
        int committedCharacterCount = e.getCommittedCharacterCount();
        if (text != null) {
            this.composedTextExists = (text.getEndIndex() - text.getBeginIndex() > committedCharacterCount);
        }
        else {
            this.composedTextExists = false;
        }
        super.processInputMethodEvent(e);
    }
    
    @Override
    protected void processFocusEvent(FocusEvent e) {
        super.processFocusEvent(e);
        if (e.isTemporary()) {
            return;
        }
        if (this.isEdited() && e.getID() == 1005) {
            InputContext inputContext = this.getInputContext();
            if (this.focusLostHandler == null) {
                this.focusLostHandler = new JFormattedTextField.FocusLostHandler();
            }
            if (inputContext != null && this.composedTextExists) {
                inputContext.endComposition();
                EventQueue.invokeLater(this.focusLostHandler);
            }
            else {
                this.focusLostHandler.run();
            }
        }
        else if (!this.isEdited()) {
            this.setValue(this.getValue(), true, true);
        }
        return;
    }
    
    @Override
    public Action[] getActions() {
        return TextAction.augmentList(super.getActions(), JFormattedTextField.defaultActions);
    }
    
    @Override
    public String getUIClassID() {
        return "FormattedTextFieldUI";
    }
    
    @Override
    public void setDocument(Document document) {
        if (this.documentListener != null && this.getDocument() != null) {
            this.getDocument().removeDocumentListener(this.documentListener);
        }
        super.setDocument(document);
        if (this.documentListener == null) {
            this.documentListener = new JFormattedTextField.DocumentHandler();
        }
        document.addDocumentListener(this.documentListener);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("FormattedTextFieldUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    private void setFormatterActions(Action[] array) {
        if (array == null) {
            if (this.textFormatterActionMap != null) {
                this.textFormatterActionMap.clear();
            }
        }
        else {
            if (this.textFormatterActionMap == null) {
                ActionMap actionMap = this.getActionMap();
                this.textFormatterActionMap = new ActionMap();
                while (actionMap != null) {
                    ActionMap parent = actionMap.getParent();
                    if (parent instanceof UIResource || parent == null) {
                        actionMap.setParent(this.textFormatterActionMap);
                        this.textFormatterActionMap.setParent(parent);
                        break;
                    }
                    actionMap = parent;
                }
            }
            for (int i = array.length - 1; i >= 0; --i) {
                Object value = array[i].getValue("Name");
                if (value != null) {
                    this.textFormatterActionMap.put(value, array[i]);
                }
            }
        }
    }
    
    private void setValue(Object o, boolean b, boolean b2) {
        Object value = this.value;
        this.value = o;
        if (b) {
            JFormattedTextField.AbstractFormatterFactory formatterFactory = this.getFormatterFactory();
            JFormattedTextField.AbstractFormatter formatter;
            if (formatterFactory != null) {
                formatter = formatterFactory.getFormatter(this);
            }
            else {
                formatter = null;
            }
            this.setFormatter(formatter);
        }
        else {
            this.setEditValid(true);
        }
        this.setEdited(false);
        if (b2) {
            this.firePropertyChange("value", value, o);
        }
    }
    
    private void setEdited(boolean edited) {
        this.edited = edited;
    }
    
    private boolean isEdited() {
        return this.edited;
    }
    
    private JFormattedTextField.AbstractFormatterFactory getDefaultFormatterFactory(Object o) {
        if ((o instanceof DateFormat)) {
            return new DefaultFormatterFactory(new DateFormatter((DateFormat)o));
        }
        if ((o instanceof NumberFormat)) {
            return new DefaultFormatterFactory(new NumberFormatter((NumberFormat)o));
        }
        if ((o instanceof Format)) {
            return new DefaultFormatterFactory(new InternationalFormatter((Format)o));
        }
        if ((o instanceof Date)) {
            return new DefaultFormatterFactory(new DateFormatter());
        }
        if ((o instanceof Number)) {
            NumberFormatter numberFormatter = new NumberFormatter();
            ((NumberFormatter)numberFormatter).setValueClass(o.getClass());
            NumberFormatter editFormat = new NumberFormatter(new DecimalFormat("#.#"));
            ((NumberFormatter)editFormat).setValueClass(o.getClass());
            return new DefaultFormatterFactory(numberFormatter, numberFormatter, editFormat);
        }
        return new DefaultFormatterFactory(new DefaultFormatter());
    }
    
    static void access$100(JFormattedTextField formattedTextField, Object o, boolean b, boolean b2) {
        formattedTextField.setValue(o, b, b2);
    }
    
    static void access$300(JFormattedTextField formattedTextField, Action[] formatterActions) {
        formattedTextField.setFormatterActions(formatterActions);
    }
    
    static void access$400(JFormattedTextField formattedTextField, boolean editValid) {
        formattedTextField.setEditValid(editValid);
    }
    
    static boolean access$500(JFormattedTextField formattedTextField) {
        return formattedTextField.isEdited();
    }
    
    static void access$600(JFormattedTextField formattedTextField, boolean edited) {
        formattedTextField.setEdited(edited);
    }
    
    static {
        defaultActions = new Action[] { new JFormattedTextField.CommitAction(), new JFormattedTextField.CancelAction() };
    }
    
    private class FocusLostHandler implements Runnable, Serializable
    {
        @Override
        public void run() {
            int focusLostBehavior = JFormattedTextField.this.getFocusLostBehavior();
            if (focusLostBehavior != 0 && focusLostBehavior != 1) {
                if (focusLostBehavior == 2) {
                    JFormattedTextField.access$100(JFormattedTextField.this, JFormattedTextField.this.getValue(), true, true);
                    return;
                }
                return;
            }
            try {
                JFormattedTextField.this.commitEdit();
                JFormattedTextField.access$100(JFormattedTextField.this, JFormattedTextField.this.getValue(), true, true);
            }
            catch (ParseException ex) {
                int n = focusLostBehavior;
                JFormattedTextField this$0 = JFormattedTextField.this;
                if (n == 1) {
                    JFormattedTextField.access$100(JFormattedTextField.this, JFormattedTextField.this.getValue(), true, true);
                }
            }
        }
    }
    
    public abstract static class AbstractFormatterFactory
    {
        public abstract JFormattedTextField.AbstractFormatter getFormatter(JFormattedTextField p0);
    }
    
    public abstract static class AbstractFormatter implements Serializable
    {
        private JFormattedTextField ftf;
        
        public void install(JFormattedTextField ftf) {
            if (this.ftf != null) {
                this.uninstall();
            }
            if ((this.ftf = ftf) != null) {
                try {
                    ftf.setText(this.valueToString(ftf.getValue()));
                }
                catch (ParseException ex) {
                    ftf.setText("");
                    this.setEditValid(false);
                }
                this.installDocumentFilter(this.getDocumentFilter());
                ftf.setNavigationFilter(this.getNavigationFilter());
                JFormattedTextField.access$300(ftf, this.getActions());
            }
        }
        
        public void uninstall() {
            if (this.ftf != null) {
                this.installDocumentFilter(null);
                this.ftf.setNavigationFilter(null);
                JFormattedTextField.access$300(this.ftf, (Action[])null);
            }
        }
        
        public abstract Object stringToValue(String p0) throws ParseException;
        
        public abstract String valueToString(Object p0) throws ParseException;
        
        protected JFormattedTextField getFormattedTextField() {
            return this.ftf;
        }
        
        protected void invalidEdit() {
            JFormattedTextField formattedTextField = this.getFormattedTextField();
            if (formattedTextField != null) {
                formattedTextField.invalidEdit();
            }
        }
        
        protected void setEditValid(boolean b) {
            JFormattedTextField formattedTextField = this.getFormattedTextField();
            if (formattedTextField != null) {
                JFormattedTextField.access$400(formattedTextField, b);
            }
        }
        
        protected Action[] getActions() {
            return null;
        }
        
        protected DocumentFilter getDocumentFilter() {
            return null;
        }
        
        protected NavigationFilter getNavigationFilter() {
            return null;
        }
        
        @Override
        protected Object clone() throws CloneNotSupportedException {
            JFormattedTextField.AbstractFormatter abstractFormatter = (JFormattedTextField.AbstractFormatter)super.clone();
            abstractFormatter.ftf = null;
            return abstractFormatter;
        }
        
        private void installDocumentFilter(DocumentFilter documentFilter) {
            JFormattedTextField formattedTextField = this.getFormattedTextField();
            if (formattedTextField != null) {
                Document document = formattedTextField.getDocument();
                if ((document instanceof AbstractDocument)) {
                    ((AbstractDocument)document).setDocumentFilter(documentFilter);
                }
                document.putProperty(DocumentFilter.class, null);
            }
        }
    }
    
    static class CommitAction extends JTextField.NotifyAction
    {
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent focusedComponent = this.getFocusedComponent();
            if ((focusedComponent instanceof JFormattedTextField)) {
                try {
                    ((JFormattedTextField)focusedComponent).commitEdit();
                }
                catch (ParseException ex) {
                    ((JFormattedTextField)focusedComponent).invalidEdit();
                    return;
                }
            }
            super.actionPerformed(actionEvent);
        }
        
        @Override
        public boolean isEnabled() {
            JTextComponent focusedComponent = this.getFocusedComponent();
            if ((focusedComponent instanceof JFormattedTextField)) {
                return JFormattedTextField.access$500((JFormattedTextField)focusedComponent);
            }
            return super.isEnabled();
        }
    }
    
    private static class CancelAction extends TextAction
    {
        public CancelAction() {
            super("reset-field-edit");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent focusedComponent = this.getFocusedComponent();
            if ((focusedComponent instanceof JFormattedTextField)) {
                JFormattedTextField formattedTextField = (JFormattedTextField)focusedComponent;
                formattedTextField.setValue(formattedTextField.getValue());
            }
        }
        
        @Override
        public boolean isEnabled() {
            JTextComponent focusedComponent = this.getFocusedComponent();
            if ((focusedComponent instanceof JFormattedTextField)) {
                return JFormattedTextField.access$500((JFormattedTextField)focusedComponent);
            }
            return super.isEnabled();
        }
    }
    
    private class DocumentHandler implements DocumentListener, Serializable
    {
        @Override
        public void insertUpdate(DocumentEvent documentEvent) {
            JFormattedTextField.access$600(JFormattedTextField.this, true);
        }
        
        @Override
        public void removeUpdate(DocumentEvent documentEvent) {
            JFormattedTextField.access$600(JFormattedTextField.this, true);
        }
        
        @Override
        public void changedUpdate(DocumentEvent documentEvent) {
        }
    }
}
